/*
          Simple Software 3D Java Rendering Engine
                  (C) Dean Camera, 2007

        dean_camera (at} fourwalledcubicle [dot> com
             http://www.fourwalledcubicle.com
*/

import java.awt.Color;

public final class Light
{
   private Point   Position;
   private float   Intensity;
   private Color   LColor;
   private float   DropOff;
   private float   DropOffSq;

   private float[] LightRGB;

   public Light(final float Intensity, final Color LColor, final float DropOff)
   {
      this(Point.ORIGIN, Intensity, LColor, DropOff);
   }

   public Light(final float Intensity, final Color LColor)
   {
      this(Point.ORIGIN, Intensity, LColor, 0);
   }

   public Light(final Point Position, final float Intensity, final Color LColor)
   {
      this(Position, Intensity, LColor, 0);
   }
   
   public Light(final Point Position, final float Intensity, final Color LColor, final float DropOff)
   {
      this.Position  = Position;
      this.Intensity = Intensity;
      this.LColor    = LColor;
      this.DropOff   = DropOff;
      this.DropOffSq = (DropOff * DropOff);

      LightRGB = new float[3];
   }

   public Point GetLocation()
   {
      return Position;
   }

   public Color GetColor()
   {
      return LColor;
   }
   
   public float GetDropOff()
   {
      return DropOff;
   }
   
   public void SetDropOff(final float DropOff)
   {
      this.DropOff   = DropOff;
      this.DropOffSq = (DropOff * DropOff);
   }
   
   public float GetIntensity()
   {
      return Intensity;
   }
   
   public void SetIntensity(final float Intensity)
   {
      this.Intensity = Intensity;
   }

   public void ApplyLight(final Point[] TriPoints, final float[] TriLocations, final float[] RGBIntensities)
   {
      final float[] LightRGB = this.LightRGB;

      final float TriPoints1X = TriPoints[1].GetX();
      final float TriPoints1Y = TriPoints[1].GetY();
      final float TriPoints1Z = TriPoints[1].GetZ();
 
      final float T1x      = (TriPoints[0].GetX() - TriPoints1X);
      final float T1y      = (TriPoints[0].GetY() - TriPoints1Y);
      final float T1z      = (TriPoints[0].GetZ() - TriPoints1Z);
     
      final float T2x      = (TriPoints1X - TriPoints[2].GetX());
      final float T2y      = (TriPoints1Y - TriPoints[2].GetY());
      final float T2z      = (TriPoints1Z - TriPoints[2].GetZ());

      final float Nx       = ((T1y * T2z) - (T1z * T2y)); 
      final float Ny       = ((T1z * T2x) - (T1x * T2z));
      final float Nz       = ((T1x * T2y) - (T1y * T2x)); 

      final float PosX     = Position.GetX();
      final float PosY     = Position.GetY();
      final float PosZ     = Position.GetZ();

      final float DotProd  = ((Nx * PosX) + (Ny * PosY) + (Nz * PosZ));
      final float NormMag  = ((Nx * Nx) + (Ny * Ny) + (Nz * Nz));
      final float LightMag = ((PosX * PosX)
                           +  (PosY * PosY)
                           +  (PosZ * PosZ));

      float FaceIntensity  = (float)(1 - (Math.acos(DotProd / Math.sqrt(NormMag * LightMag)) / Math.PI));

      if (DropOff > 0)
      {
         final float TDistSq = (Math.abs(TriLocations[0] - PosX)
                             +  Math.abs(TriLocations[1] - PosY)
                             +  Math.abs(TriLocations[2] - PosZ));

         if (TDistSq > DropOffSq)
            FaceIntensity *= (DropOff / Math.sqrt(TDistSq));
      }
  
      FaceIntensity *= Intensity;

      LColor.getRGBColorComponents(LightRGB);
      RGBIntensities[0] += (LightRGB[0] * FaceIntensity);
      RGBIntensities[1] += (LightRGB[1] * FaceIntensity);
      RGBIntensities[2] += (LightRGB[2] * FaceIntensity);
   }
}

